/**************************************************************************
 *	Copyright (C) 2004-2005  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_node.hpp"
#include "gl_util.hpp"

//TODO: Remove these:
bool labelIni	= false;
GLfloat* nodeLabel;

/**
	This function draws a node.  A node is, the IP block/domain whatever, 
	label and the host tree.  It replaces a lot of the scan-draw code
 */
void FeNode::draw(GLenum mode)
{
	drawNodeLabel();
	int hostCount = (int)m_Hosts.size();
	float slice = 0;

	if(hostCount > 0)
	{
		slice = 360.0f/(float)hostCount;
	}

	m_Radius = (((0.05f)*10*hostCount)/TWOPI);

	float j = 0;

	glColor3f(0.0f,0.0f,1.0f);

	for(vector<HostDraw*>::iterator i = m_Hosts.begin(); i != m_Hosts.end(); ++i)
	{
		glPushMatrix();

		glRotatef(slice*j++,0,1.0f,0);

		glColor3f(0.0f,0.0f,1.0f);
		glDisable(GL_LIGHTING);
		glBegin(GL_LINES);
			glVertex3f( 0, 0, 0);
			glVertex3f( 0, -0.4f,m_Radius);
		glEnd();
		glEnable(GL_LIGHTING);

		glTranslatef(0,-0.4f,m_Radius);
		(*i)->draw();
		glPopMatrix();
	}
	

}

void FeNode::addHost(HostDraw* host)
{
	if(host == NULL)
		return;

	m_Hosts.push_back(host);
	
	if(m_Label.size() == 0)
	{
		string tmp = host->getIp();
		m_Label = tmp.substr(0, tmp.find("."));
		m_Label.append(".*");
	}
}

FeNode::FeNode()
{
	this->m_texLabel = 0;
}

FeNode::FeNode(string label)
{
	this->m_texLabel = 0;
	m_Label = label;
}

FeNode::~FeNode()
{
	while(!m_Hosts.empty())
	{
		delete *(m_Hosts.begin());
		m_Hosts.erase(m_Hosts.begin());
	}
}

float FeNode::getRadius()
{
	return m_Radius;
}

void FeNode::drawNodeLabel()
{
	if(!labelIni)
	{
		const float labelWidth = 0.03f;
		const float rad = 0.16f;
		const int	n = 15;
		const int slice = 60/n;

		float x = 0;
		float z = 0;
		float mag = 0;
		int s = 0;
	
		nodeLabel = new GLfloat[16*n];
		for(float j = 0; j < n ; ++j)
		{
			x=rad*cos((float)((Pi/180)*j*slice));
			z=rad*sin((float)((Pi/180)*j*slice));
			mag = sqrt(x*x + z*z);
			*(nodeLabel + s++) = (float)(1 - j/n);
			*(nodeLabel + s++) = 1.0f;
			*(nodeLabel + s++) = (float)x/mag;
			*(nodeLabel + s++) = 0.0f;
			*(nodeLabel + s++) = (float)z/mag;
			*(nodeLabel + s++) = (float)x;
			*(nodeLabel + s++) = 0.0f;
			*(nodeLabel + s++) = (float)z;
			*(nodeLabel + s++) = (float)(1 - j/n);
			*(nodeLabel + s++) = 0.0f;
			*(nodeLabel + s++) = (float)x/mag;
			*(nodeLabel + s++) = 0.0f;
			*(nodeLabel + s++) = (float)z/mag;
			*(nodeLabel + s++) = (float)x;
			*(nodeLabel + s++) = labelWidth;
			*(nodeLabel + s++) = (float)z;
		}
		labelIni = true;
	}

	if(m_texLabel == 0)
	{
		SDL_Surface* tmpSurf = NULL;
		if(m_Config == NULL){
			printf("FeNode::drawLabel -) ERROR m_Config is NULL\n");
			return;
		}
		if(m_Config->m_glUtil == NULL){
			printf("FeNode::drawLabel -) ERROR m_Config->m_glUtil is NULL\n");
			return;	
		}
		tmpSurf = m_Config->m_glUtil->createTextSurfaceGL(m_Label.c_str(), 256, 32, 30, TRUE);
		if(tmpSurf == NULL)
			return;
		glGenTextures(1, &m_texLabel);
		glBindTexture(GL_TEXTURE_2D, m_texLabel);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture
		glTexImage2D(GL_TEXTURE_2D, 0, 3, tmpSurf->w, tmpSurf->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tmpSurf->pixels);
		SDL_FreeSurface(tmpSurf);
	}

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, m_texLabel);
	glInterleavedArrays(GL_T2F_N3F_V3F, 0, nodeLabel);

	glPushMatrix();
		glTranslatef(0.0f, 0.0f, -0.08f);
		glRotatef(60, 0.0f, -1.0f, 0.0f);
		glDrawArrays(GL_QUAD_STRIP, 0, 30);
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);

}

