/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_parser.hpp"

ScanParser::ScanParser()
{
	m_Parser = NULL;
	m_errHandler = (ErrorHandler*) new HandlerBase();

};

ScanParser::~ScanParser()
{
	delete m_errHandler;
	delete m_Parser;

	XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Terminate();
	
	while(!this->m_Parsers.empty())
	{
		delete *(m_Parsers.begin());
		m_Parsers.erase(m_Parsers.begin());
	}
}

bool ScanParser::initParser()
{
	try
	{
		XMLPlatformUtils::Initialize();
	}
	catch(const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch)
	{
		char* message = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage());
		cerr<< "Error during initialization! :\n"
		     << message << "\n";

		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&message);
		return FALSE;
	}

	m_Parser = new XercesDOMParser();
	m_Parser->setValidationScheme(XercesDOMParser::Val_Always);    // optional.
	m_Parser->setDoNamespaces(true);    // optional

	m_Parser->setErrorHandler(m_errHandler);

	return TRUE;

}

void ScanParser::addParser(FeParser *parser)
{
	this->m_Parsers.push_back(parser);
}

bool ScanParser::parseFile(string filename)
{
	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument* doc = NULL;

	try
	{
		m_Parser->resetDocumentPool();
		m_Parser->parse(filename.c_str());
	}
	catch (const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch)
	{
		char* message = XMLString::transcode(toCatch.getMessage());
		cerr<< "Exception message is: \n"
			<< message << "\n";
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&message);
		return FALSE;
	}
	catch (const DOMException& toCatch)
	{
		char* message = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.msg);
		cerr<< "Exception message is: \n"
			<< message << "\n";
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&message);
		return FALSE;
	}
	catch (...)
	{
		cerr<< "Unexpected Exception \n" ;
		return FALSE;
	}

	if(m_Parser == NULL)
	{
		return FALSE;
	}

	doc  = m_Parser->getDocument();

	if(!doc)
	{
		return FALSE;
	}
	
	vector<FeParser*>::const_iterator iter = m_Parsers.begin();

	FeParser* tmp = NULL;

	while(iter != m_Parsers.end())
	{
		tmp = *iter;	

		if(tmp->isUsable(doc))
		{
			tmp->setScan(m_Scan);
			tmp->setConfig(m_Config);
				return tmp->Parse();
		}
		++iter;
	}

	// Iterate over all parsers and give them this doc looking for a
	// true result


	return FALSE;
}

