/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef _H_F3_FE_PARSER
#define _H_F3_FE_PARSER

#include <fstream>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>

#include "scan_draw.hpp"

XERCES_CPP_NAMESPACE_USE
using namespace std;

/**
 *	All parsers that work with scan logs and add information, 
 * to the host hirarchy must extend this class and override
 * the necessary methods. 
 * 
 * \brief 
 *	Base class for all classes that parse XML Log 
 * files generated by security tools
 *
 * \version 0.7-43
 * inital version
 *
 * \date 02-14-2004
 *
 * \author Stou
 *
 * \par license GPL
 * 
 * \todo 
 *
 *
 */
class FeParser
{

public: // Construction / Destruction
	FeParser() { };
	virtual ~FeParser() { };

public:

	/**
	*	Tests if the parser instance recognizes the
	* XML document for parsing.
	*
	*	If the document is recognized for processing,
	* it will be set internaly for processing.
	*
	* \param doc An XML document
	* \return true if the parser recognizes this document
	*			false otherwise.
	*/
	virtual bool isUsable(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument* doc) = 0;

	/**
	*	Parses the currently selected document.
	*
	* \return true if the parse is successfuly	false otherwise.
	*/
	virtual bool Parse() = 0;

public: // Setter/Getters
	void setConfig(XMLConfig* cfg) { m_Config = cfg; };
	void setScan(ScanDraw* scan) { m_Scan = scan; };

protected: // Data members
	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument* m_xmlDoc;
	ScanDraw* m_Scan;
	XMLConfig* m_Config;

};


/**
 * \brief 
 *	The XML Log parser manager, it delegates 
 * parsing to the 'correct' parser upon loading
 * of a scan log.
 *
 * \version 0.7-43
 * inital version
 *
 * \date 02-14-2004
 *
 * \author Stou
 *
 * \par license GPL
 * 
 * \todo 
 *
 */
class ScanParser
{
public: // Construction/Destruction
	ScanParser();
	~ScanParser();
public: // Setter/Getters
	void setConfig(XMLConfig* cfg) { m_Config = cfg; };
	void setScan(ScanDraw* scan) { m_Scan = scan; };


public: // XML Parsing functionality
	bool initParser();
	bool parseFile(string filename);

public: // Parser managing functions
	void addParser(FeParser* parser);


private:
	XercesDOMParser* m_Parser;
	ErrorHandler* m_errHandler;
	XMLConfig* m_Config;
	ScanDraw* m_Scan;

	vector<FeParser*>	m_Parsers; // Stores a list of available parsers;
	bool				m_MultiParse; // true-keep giving to other parsers
};


#endif //_H_F3_FE_PARSER

