/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "png_screen.hpp"

int ss_id = 0;

PNGScreen::PNGScreen()
{
	ss_id = 0;
	m_png_ptr = NULL;
	m_info_ptr = NULL;
}

bool PNGScreen::init()
{
	char filename[25];

	sprintf(filename, "fe3d-%s-%i.%03d.png", PRODUCT_VERSION, BUILD_NUM, ss_id++);

	glGetIntegerv(GL_VIEWPORT, viewPort);

	pixels = new GLubyte[viewPort[2]*viewPort[3]*3];
//	printf("%s\n", gluErrorString(glGetError()));
	glReadBuffer(GL_FRONT);
	glReadPixels(0, 0, viewPort[2], viewPort[3], GL_RGB, GL_UNSIGNED_BYTE, pixels);
	m_fp = fopen(filename, "wb");

    if (!m_fp)
    {
       return FALSE;
    }


    m_png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, 
										NULL,
										pngError,
										pngWarning);
    if (!m_png_ptr)
       return FALSE;

    m_info_ptr = png_create_info_struct(m_png_ptr);

    if (!m_info_ptr)
    {
       png_destroy_write_struct(&m_png_ptr, (png_infopp)NULL);
       fclose(m_fp);
	   return FALSE;
    }

	if (setjmp(png_jmpbuf(m_png_ptr)))
    {
       png_destroy_write_struct(&m_png_ptr, &m_info_ptr);
       fclose(m_fp);
       return FALSE;
    }

	png_init_io(m_png_ptr, m_fp);

	//png_set_write_status_fn(m_png_ptr, writeRowCallback);
	
	return TRUE;
}

void PNGScreen::doStuff()
{
	if(!m_info_ptr)
	{
		return;
	}
	png_text  text[3];

	png_bytep *screen = new png_bytep[viewPort[3]];
	png_bytep *scrn	 = screen;

	for(int i = 0; i < viewPort[3] ; ++i)
	{
		(*scrn) = pixels + (viewPort[2]*3)*(viewPort[3] - i);
		++scrn;
	}

    text[0].compression = PNG_TEXT_COMPRESSION_NONE;
    text[0].key = "Title";
    text[0].text = "fe3d Screenshot";
	text[0].text_length = 15;

	text[1].compression = PNG_TEXT_COMPRESSION_NONE;
    text[1].key = "Source";
	text[1].text = (png_charp)m_Source.c_str();
	text[1].text_length = m_Source.size();

	text[2].compression = PNG_TEXT_COMPRESSION_NONE;
    text[2].key = "Software";
    text[2].text = (png_charp)m_Software.c_str();
	text[2].text_length = m_Software.size();

	png_time t;
	png_convert_from_time_t(&t, time(0));
	png_set_tIME(m_png_ptr, m_info_ptr, &t);

    png_set_text(m_png_ptr, m_info_ptr, text, 3);

	png_set_compression_level(m_png_ptr, Z_BEST_COMPRESSION);
	png_set_IHDR(m_png_ptr, m_info_ptr, viewPort[2], viewPort[3], 8,
					PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
					PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);

	png_write_info(m_png_ptr, m_info_ptr);
	png_write_image(m_png_ptr, screen);
	png_write_end(m_png_ptr, m_info_ptr);

}

bool PNGScreen::kill()
{
	if((m_png_ptr) && (m_info_ptr))
		png_destroy_write_struct(&m_png_ptr, &m_info_ptr);
	fclose(m_fp);
	delete pixels;
	return TRUE;
}

void PNGScreen::initText()
{
	char str[255];

	m_Source.append((char*)glGetString(GL_VENDOR));
	m_Source.append(" ");
	m_Source.append((char*)glGetString(GL_RENDERER));
	m_Source.append(" ");
	m_Source.append((char*)glGetString(GL_VERSION));
	
	m_Software.append("fe3d ");
	m_Software += PRODUCT_VERSION;
	m_Software.append(" build ");
	sprintf(str, "%d", BUILD_NUM );
	m_Software += str;
}

void PNGScreen::writeRowCallback(png_structp png_ptr, png_uint_32 row, int pass)
{
	// We don't need this


}

void PNGScreen::pngError(png_structp str, png_const_charp msg)
{
    fprintf(stderr, "writepng libpng error: %s\n", msg);
}

void PNGScreen::pngWarning(png_structp str, png_const_charp msg)
{
    fprintf(stderr, "writepng libpng warning: %s\n", msg);
}

